__author__ = "Lukas Leufen"
__date__ = "2022-07-05"

from . import join, toar_data_v2

import requests
import pandas as pd
from .data_loader import EmptyQueryResult
import xarray as xr


def download_toar(station, toar_stats, sampling, data_origin, window_dim, time_dim, target_dim):
    try:
        # load data from toar-data (v2)
        df_toar, meta_toar = toar_data_v2.download_toar(station, toar_stats, sampling=sampling, data_origin=data_origin)
    except (AttributeError, EmptyQueryResult, KeyError, requests.ConnectionError, ValueError, IndexError):
        df_toar, meta_toar = None, None

    try:
        # load join data (toar-data v1)
        df_join, meta_join = join.download_join(station_name=station, stat_var=toar_stats, sampling=sampling,
                                                data_origin=data_origin)
    except (AttributeError, EmptyQueryResult, KeyError, requests.ConnectionError, ValueError, IndexError):
        df_join, meta_join = None, None

    # merge both data sources with priority on toar-data v2
    if df_toar is not None and df_join is not None:
        df_merged = merge_toar_join(df_toar, df_join, sampling)
        meta_merged = meta_toar
    else:
        df_merged = df_toar if df_toar is not None else df_join
        meta_merged = meta_toar if df_toar is not None else meta_join
    xarr_merged = xr.DataArray(df_merged, dims=[time_dim, target_dim])
    xarr_merged = xarr_merged.expand_dims({window_dim: [0]})
    return xarr_merged, meta_merged


def merge_toar_join(df_toar, df_join, sampling):
    start_date = min([df_toar.index.min(), df_join.index.min()])
    end_date = max([df_toar.index.max(), df_join.index.max()])
    freq = {"hourly": "1H", "daily": "1d"}.get(sampling)
    full_time = pd.date_range(start_date, end_date, freq=freq)
    full_data = df_toar.reindex(full_time)
    full_data.update(df_join, overwrite=False)
    return full_data
